import yaml

configDefaults = {
    "sizes": [
        "1024x768",
        "1920x1280",
        "1280x720"
    ],
    "border": {
        "size": 20,
        "color": "000"
    },
    "grid": {
        "1024x768": [3, 4],
        "": [5, 4]
    },
    "inclusions": 5
}


def mergedict(a, b, path=None):
    """
    Dictionary merge
    http://stackoverflow.com/a/7205107/744180
    """

    path = [] if path is None else path
    for key in b:
        if key in a:
            if isinstance(a[key], dict) and isinstance(b[key], dict):
                mergedict(a[key], b[key], path + [str(key)])
        else:
            a[key] = b[key]
    return a


def load(configPath):
    """
    Load configuration
    """

    with open(configPath) as configFile:
        raw = configFile.read()
        loadedConfig = yaml.safe_load(raw)
        return mergedict(loadedConfig, configDefaults)



from PIL import Image as Img
from PIL import ImageOps


class Image(object):
    """
    Image object
    """

    def __init__(self, filePath):
        self.file = filePath
        _f = self.rawFile
        self.im = Img.open(_f)
        self.size = self.im.size
        _f.close()

    def resize(self, width, height):
        """
        Resize the image to the specified size
        Resizes with croping/scaling
        """
        with self as im:
            return ImageOps.fit(im, (width, height), centering=(0.5, 0.5))

    def ratioChange(self, rFrom, rTo):
        return float(rTo) / float(rFrom)

    def __enter__(self):
        self._file = self.rawFile
        return Img.open(self._file)

    def __exit__(self, *sp):
        self._file.close()

    @property
    def rawFile(self):
        return open(self.file, 'rb')

    @property
    def aspect(self):
        return float(self.im.size[0]) / float(self.im.size[1])

    @property
    def portrait(self):
        return self.aspect < 1



from PIL import Image as Img


class TileRenderer(object):
    def __init__(self, grid, tileSize, cellSize, border, filePath):
        self.tileSize = tileSize
        self.cellSize = cellSize
        self.im = Img.new("RGB", tileSize)
        self.filePath = filePath

        for i, tile in enumerate(grid.grid):
            sizeX = (cellSize[0]*tile['width'] - border['size'])
            sizeY = (cellSize[1]*tile['height'] - border['size'])

            r = tile['image'].resize(sizeX, sizeY)
            box = (
                int(cellSize[0]*tile['x']+border['size']/2),
                int(cellSize[1]*tile['y']+border['size']/2)
            )
            self.im.paste(r, box)

        self.im.save(filePath, 'JPEG')



import random


class Grid(object):
    """
    Grid object (Single tile)
    """

    pickedImages = []
    grid = []

    def __init__(self, images, gridSize):
        self.images = images
        random.shuffle(self.images)
        self.gridSize = gridSize
        self.grid = []

        for im in self.images:
            pos = self.getImagePosition(im)
            if not pos:
                pos = self.getImagePosition(im, small=True)
                if not pos:
                    break

            self.grid.append({
                "x": pos[0],
                "y": pos[1],
                "width": pos[2],
                "height": pos[3],
                "image": im
            })

    def getImageSize(self, im, small=False):
        if im.portrait:
            height = 2
            width = 1
        else:
            if random.randint(0, 10) <= 2 and not small:
                height = 2
                width = 2
            else:
                height = 1
                width = 1

        return width, height

    def getImagePosition(self, im, small=False):
        """
        Calculate the position by checking against the status of the grid
        """
        width, height = self.getImageSize(im, small)
        for x, y in self.gridCoords:
            # Check coords are within grid
            if not all(
                [x+xPos <= self.gridSize[0] for xPos in range(0, width+1)]
            ) or not all(
                [y+yPos <= self.gridSize[1] for yPos in range(0, height+1)]
            ):
                continue

            if any(i in self.occupiedCoords
                   for i in self.imageGridCoords(x, y, width, height)):
                continue

            return x, y, width, height

    def imageGridCoords(self, xBase, yBase, width, height):
        """
        Return a list of positions occupied by an image
        """
        coords = []
        for xPos in range(0, width):
            for yPos in range(0, height):
                coords.append([
                    xPos+xBase,
                    yPos+yBase
                ])
        return coords

    @property
    def full(self):
        """
        Does the grid have any free space on it
        """

        return all(c in self.occupiedCoords for c in self.gridCoords)

    @property
    def occupiedCoords(self):
        """
        Return all coords with an image in
        """

        coords = []
        for im in self.grid:
            for x, y in self.imageGridCoords(
                im['x'],
                im['y'],
                im['width'],
                im['height']
            ):
                coords.append([x, y])
        return coords

    @property
    def gridCoords(self):
        """
        Return a list of all possible grid coords
        """
        coords = []
        for x in range(0, self.gridSize[0]):
            for y in range(0, self.gridSize[1]):
                coords.append([x, y])
        return coords



from argparse import ArgumentParser
import logging
import tilepaper.tiler
import tilepaper.config


def run():
    p = ArgumentParser()
    p.add_argument('--source', required=True)
    p.add_argument('--destination', required=True)
    p.add_argument('--config', required=True)
    p.add_argument('--quiet', action="store_true")
    p.add_argument('--verbose', action="store_true")
    options = p.parse_args()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)
    elif options.quiet:
        logging.basicConfig(level=logging.WARNING)
    else:
        logging.basicConfig(level=logging.INFO)

    config = tilepaper.config.load(options.config)
    t = tilepaper.tiler.generator()
    t.process(
        source=options.source,
        dest=options.destination,
        **config
    )


if __name__ == "__main__":
    run()



from setuptools import setup, find_packages
import tilepaper

requirements = ['PyYAML==3.11', 'filemagic==1.6', 'Pillow==3.0.0']

setup(name='tilepaper',
      version=tilepaper.__version__,
      author='Sam Rudge',
      author_email='sam@codesam.co.uk',
      packages=find_packages(),
      install_requires=requirements,
      include_package_data=True,
      zip_safe=False,
      url="https://github.com/samarudge/tilepaper",
      entry_points="""
[console_scripts]
tilepaper = tilepaper.cli:run
""",
      classifiers=[
          'Environment :: Console',
          'Intended Audience :: End Users/Desktop',
          'License :: OSI Approved :: MIT License',
          'Topic :: Multimedia :: Graphics',
          'Topic :: Utilities'
      ]
      )



import magic
import os
import logging
from tilepaper.image import Image
from tilepaper.grid import Grid
from tilepaper.renderer import TileRenderer


class ConfigException(Exception):
    None


class generator(object):
    imageTypes = ['image/jpeg', 'image/png']

    def process(self, source, dest, **config):
        self.source = os.path.abspath(
            os.path.expanduser(source)
        )
        self.dest = os.path.abspath(
            os.path.expanduser(dest)
        )
        self.config = config

        for d in [("Source", self.source), ("Destination", self.dest)]:
            if not os.path.isdir(d[1]):
                raise ConfigException("%s directory not found!" % d[0])

        images = self.findImages(self.source)
        logging.info("Found %d images" % len(images))

        for size in config['sizes']:
            logging.info("Generating tile format for %s" % size)
            gridSize = config['grid'][size]\
                if size in config['grid'] else config['grid']['default']
            tiles = self.generateTiles(
                images,
                gridSize,
                inclusions=config['inclusions']
            )
            logging.info("Generated %d tile layouts" % len(tiles))
            logging.info("Rendering")
            self.renderTiles(tiles, size, gridSize)

    def findImages(self, directory):
        """
        Recursive scan directory to find images
        """

        logging.debug("Finding images in %s" % directory)
        images = []
        with magic.Magic(flags=magic.MAGIC_MIME_TYPE) as m:
            for fileName in os.listdir(directory):
                fullPath = os.path.join(directory, fileName)

                if os.path.isdir(fullPath):
                    images += self.findImages(fullPath)
                else:
                    mimeType = m.id_filename(fullPath)
                    if mimeType in self.imageTypes:
                        images.append(Image(fullPath))
                    else:
                        logging.debug(
                            "Skipping file %s with mime %s"
                            % (fullPath, mimeType)
                        )
        return images

    def generateTiles(self, images, gridSize, inclusions):
        """
        Generate the tile formats
        """

        tileId = 1
        imageUses = {}
        tiles = []
        while True:
            avaliableImages = [
                im for im in images
                if im.file not in imageUses.keys() or
                imageUses[im.file] < inclusions
            ]

            if len(avaliableImages) == 0:
                break

            logging.debug(
                "Tile %06d with %d images remaining"
                % (tileId, len(avaliableImages))
            )
            grid = Grid(avaliableImages, gridSize)

            if grid.full:
                tiles.append({"id": tileId, "grid": grid})
                tileId += 1
            else:
                logging.debug("Tile not full, skipping")

            for cell in grid.grid:
                im = cell['image']
                if im.file not in imageUses:
                    imageUses[im.file] = 1
                else:
                    imageUses[im.file] += 1

        return tiles

    def renderTiles(self, tiles, size, gridSize):
        """
        Render out the tiles
        """

        tileSize = [int(c) for c in size.split('x')]
        cellSize = [
            int(float(tileSize[0])/float(gridSize[0])),
            int(float(tileSize[-1])/float(gridSize[-1]))
        ]

        renderDir = os.path.join(self.dest, size)
        if not os.path.isdir(renderDir):
            os.mkdir(renderDir)

        for tile in tiles:
            logging.debug("Rendering tile %d of %d" % (tile['id'], len(tiles)))
            filePath = os.path.join(
                renderDir,
                "%06d.jpg" % tile['id']
            )
            TileRenderer(tile['grid'],
                         tileSize,
                         cellSize,
                         self.config['border'],
                         filePath)
